@@ -21,15 +21,17 @@ from theanolm.backend import test_value
 from theanolm.network.architecture import Architecture
 from theanolm.network.networkinput import NetworkInput
 from theanolm.network.projectionlayer import ProjectionLayer
-from theanolm.network.tanhlayer import TanhLayer
+from theanolm.network.fullyconnectedlayer import FullyConnectedLayer
+from theanolm.network.additionlayer import AdditionLayer
 from theanolm.network.grulayer import GRULayer
 from theanolm.network.lstmlayer import LSTMLayer
-from theanolm.network.highwaytanhlayer import HighwayTanhLayer
+from theanolm.network.highwaylayer import HighwayLayer
 from theanolm.network.glulayer import GLULayer
 from theanolm.network.softmaxlayer import SoftmaxLayer
 from theanolm.network.hsoftmaxlayer import HSoftmaxLayer
 from theanolm.network.dropoutlayer import DropoutLayer
 from theanolm.network.bidirectionallayer import BidirectionalLayer
+from theanolm.network.samplingoutputlayer import SamplingOutputLayer
 
 def create_layer(layer_options, *args, **kwargs):
     """Constructs one of the Layer classes based on a layer definition.
@@ -41,16 +43,18 @@ def create_layer(layer_options, *args, **kwargs):
     layer_type = layer_options['type']
     if layer_type == 'projection':
         return ProjectionLayer(layer_options, *args, **kwargs)
-    elif layer_type == 'tanh':
-        return TanhLayer(layer_options, *args, **kwargs)
+    elif layer_type == 'fc' or layer_type == 'tanh':
+        return FullyConnectedLayer(layer_options, *args, **kwargs)
+    elif layer_type == 'add':
+        return AdditionLayer(layer_options, *args, **kwargs)
     elif layer_type == 'lstm':
         return LSTMLayer(layer_options, *args, **kwargs)
     elif layer_type == 'gru':
         return GRULayer(layer_options, *args, **kwargs)
     elif layer_type == 'blstm' or layer_type == 'bgru':
         return BidirectionalLayer(layer_options, *args, **kwargs)
-    elif layer_type == 'highwaytanh':
-        return HighwayTanhLayer(layer_options, *args, **kwargs)
+    elif layer_type == 'highway' or layer_type == 'highwaytanh':
+        return HighwayLayer(layer_options, *args, **kwargs)
     elif layer_type == 'glu':
         return GLULayer(layer_options, *args, **kwargs)
     elif layer_type == 'softmax':
@@ -194,7 +198,7 @@ class Network(object):
             self.layers[layer.name] = layer
         self.output_layer = self.layers[architecture.output_layer]
         num_params = sum(layer.num_params() for layer in self.layers.values())
-        logging.debug("Total number of parameters: %d", num_params)
+        logging.debug("Total number of model parameters: %d", num_params)
 
         # This list will be filled by the recurrent layers to contain the
         # recurrent state outputs, for doing forward passes one step at a time.
@@ -327,7 +331,10 @@ class Network(object):
             raise ValueError("Invalid noise distribution requested: `{}'"
                              .format(type))
 
-        if sharing is None:
+        if not isinstance(output_layer, SamplingOutputLayer):
+            self._noise_sample = None
+            self._noise_sample_logprobs = None
+        elif sharing is None:
             self._noise_sample, self._noise_sample_logprobs = \
                 output_layer.get_sample_tensors(distribution)
         elif sharing == 'seq':
@@ -485,8 +492,8 @@ class Network(object):
 
         if (self._noise_sample is None) or \
            (self._noise_sample_logprobs is None):
-            raise RuntimeError(
-                "Trying to read the noise sample before defining sampling.")
+            raise InputError(
+                "The output layer does not support sampling noise words.")
 
         return self._noise_sample, self._noise_sample_logprobs
 
