@@ -46,8 +46,6 @@ class RMSPropNesterovOptimizer(BasicOptimizer):
 
         self._params = Parameters()
         for path, param in network.get_variables().items():
-            self._params.add(path + '_gradient',
-                             numpy.zeros_like(param.get_value()))
             # Initialize mean squared gradient to ones, otherwise the first
             # update will be divided by close to zero.
             self._params.add(path + '_mean_sqr_gradient',
@@ -68,34 +66,37 @@ class RMSPropNesterovOptimizer(BasicOptimizer):
 
         super().__init__(optimization_options, network, *args, **kwargs)
 
-    def _gradient_update_exprs(self):
+    def _get_param_updates(self, alpha):
+        """Returns Theano expressions for updating the model parameters and any
+        additional parameters required by the optimizer.
+
+        :type alpha: Variable
+        :param alpha: a scale to be applied to the model parameter updates
+
+        :rtype: iterable over pairs (shared variable, new expression)
+        :returns: expressions how to update the optimizer parameters
+        """
+
         result = []
-        for path, gradient_new in zip(self.network.get_variables(),
-                                      self._gradient_exprs):
-            gradient = self._params[path + '_gradient']
-            ms_gradient = self._params[path + '_mean_sqr_gradient']
-            ms_gradient_new = \
-                self._gamma * ms_gradient + \
-                (1.0 - self._gamma) * tensor.sqr(gradient_new)
-            result.append((gradient, gradient_new))
-            result.append((ms_gradient, ms_gradient_new))
-        return result
+        deltas = dict()
+        for path, gradient in zip(self.network.get_variables(),
+                                  self._gradients):
+            ms_gradient_old = self._params[path + '_mean_sqr_gradient']
+            ms_gradient = \
+                self._gamma * ms_gradient_old + \
+                (1.0 - self._gamma) * tensor.sqr(gradient)
+            result.append((ms_gradient_old, ms_gradient))
 
-    def _model_update_exprs(self, alpha):
-        updates = dict()
-        for path, param in self.network.get_variables().items():
-            gradient = self._params[path + '_gradient']
-            ms_gradient = self._params[path + '_mean_sqr_gradient']
             rms_gradient = tensor.sqrt(ms_gradient + self._epsilon)
-            updates[path] = -gradient / rms_gradient
-        self._normalize(updates)
+            deltas[path] = -gradient / rms_gradient
+        self._normalize(deltas)
 
         result = []
-        for path, param in self.network.get_variables().items():
-            update = updates[path]
-            velocity = self._params[path + '_velocity']
-            velocity_new = self._momentum * velocity + alpha * update
-            param_new = param + self._momentum * velocity_new + alpha * update
-            result.append((velocity, velocity_new))
-            result.append((param, param_new))
+        for path, param_old in self.network.get_variables().items():
+            delta = deltas[path]
+            velocity_old = self._params[path + '_velocity']
+            velocity = self._momentum * velocity_old + alpha * delta
+            param = param_old + self._momentum * velocity + alpha * delta
+            result.append((velocity_old, velocity))
+            result.append((param_old, param))
         return result
