@@ -12,8 +12,8 @@ import h5py
 import numpy
 import theano
 
+from theanolm.backend import IncompatibleStateError
 from theanolm.parsing import ShufflingBatchIterator, LinearBatchIterator
-from theanolm.exceptions import IncompatibleStateError, NumberError
 from theanolm.training.stoppers import create_stopper
 
 class Trainer(object):
@@ -48,8 +48,7 @@ class Trainer(object):
 
         self._vocabulary = vocabulary
 
-        print("Computing class unigram probabilities and the number of "
-              "mini-batches in training data.")
+        print("Computing the number of mini-batches in training data.")
         linear_iter = LinearBatchIterator(
             training_files,
             vocabulary,
@@ -58,21 +57,14 @@ class Trainer(object):
             map_oos_to_unk=True)
         sys.stdout.flush()
         self._updates_per_epoch = 0
-        class_counts = numpy.zeros(vocabulary.num_classes(), dtype='int64')
-        for word_ids, _, mask in linear_iter:
+        for _, _, _ in linear_iter:
             self._updates_per_epoch += 1
-            word_ids = word_ids[mask == 1]
-            class_ids = vocabulary.word_id_to_class_id[word_ids]
-            numpy.add.at(class_counts, class_ids, 1)
         if self._updates_per_epoch < 1:
             raise ValueError("Training data does not contain any sentences.")
         logging.debug("One epoch of training data contains %d mini-batch "
                       "updates.",
                       self._updates_per_epoch)
-        total_count = class_counts.sum()
-        self.class_prior_probs = class_counts.astype(theano.config.floatX)
-        if total_count > 0:
-            self.class_prior_probs /= total_count
+        self.class_prior_probs = vocabulary.get_class_probs()
         logging.debug("Class unigram log probabilities are in the range [%f, "
                       "%f].",
                       numpy.log(self.class_prior_probs.min()),
@@ -497,10 +489,6 @@ class Trainer(object):
             return  # We don't have to validate now.
 
         perplexity = self._scorer.compute_perplexity(self._validation_iter)
-        if numpy.isnan(perplexity) or numpy.isinf(perplexity):
-            raise NumberError("Validation set perplexity computation resulted "
-                              "in a numerical error.")
-
         self._local_perplexities.append(perplexity)
         if len(self._local_perplexities) == 1:
             logging.debug("[%d] First validation sample, perplexity %.2f.",
